@@ -32,6 +32,7 @@ require ZoneMinder::Base;
 require ZoneMinder::Object;
 require ZoneMinder::Storage;
 require ZoneMinder::Frame;
+require ZoneMinder::Monitor;
 require Date::Manip;
 require File::Find;
 require File::Path;
@@ -68,8 +69,8 @@ $serial = $primary_key = 'Id';
   SecondaryStorageId
   Name
   Cause
-  StartTime
-  EndTime
+  StartDateTime
+  EndDateTime
   Width
   Height
   Length
@@ -111,8 +112,8 @@ sub Time {
     $_[0]{Time} = $_[1];
   }
   if ( ! defined $_[0]{Time} ) {
-    if ( $_[0]{StartTime} ) {
-      $_[0]{Time} = Date::Parse::str2time( $_[0]{StartTime} );
+    if ( $_[0]{StartDateTime} ) {
+      $_[0]{Time} = Date::Parse::str2time( $_[0]{StartDateTime} );
     }
   }
   return $_[0]{Time};
@@ -349,47 +350,58 @@ sub GenerateVideo {
   return;
 } # end sub GenerateVideo
 
+# Note about transactions, this function may be called with rows locked and hence in a transaction.
+# So we will detect if we are in a transaction, and if not, start one.  We will NOT do rollback or 
+# commits unless we started the transaction.
+
 sub delete {
   my $event = $_[0];
 
+  if ( !$event->canEdit() ) {
+    Warning('No permission to delete event.');
+    return 'No permission to delete event.';
+  }
+
   my $in_zmaudit = ( $0 =~ 'zmaudit.pl$');
 
   if ( ! $in_zmaudit ) {
-    if ( ! ( $event->{Id} and $event->{MonitorId} and $event->{StartTime} ) ) {
+    if ( ! ( $event->{Id} and $event->{MonitorId} and $event->{StartDateTime} ) ) {
       # zmfilter shouldn't delete anything in an odd situation. zmaudit will though.
       my ( $caller, undef, $line ) = caller;
-      Warning("$0 Can't Delete event $event->{Id} from Monitor $event->{MonitorId} StartTime:".
-        (defined($event->{StartTime})?$event->{StartTime}:'undef')." from $caller:$line");
+      Warning("$0 Can't Delete event $event->{Id} from Monitor $event->{MonitorId} StartDateTime:".
+        (defined($event->{StartDateTime})?$event->{StartDateTime}:'undef')." from $caller:$line");
       return;
     }
     if ( !($event->Storage()->Path() and -e $event->Storage()->Path()) ) {
-      Warning('Not deleting event because storage path doesn\'t exist');
+      Warning('Not deleting event because storage path ('.$event->Storage()->Path().') doesn\'t exist');
       return;
     }
   }
 
   if ( $$event{Id} ) {
     # Need to have an event Id if we are to delete from the db.  
-    Info("Deleting event $event->{Id} from Monitor $event->{MonitorId} StartTime:$event->{StartTime} from ".$event->Path());
+    Info("Deleting event $event->{Id} from Monitor $event->{MonitorId} StartDateTime:$event->{StartDateTime} from ".$event->Path());
     $ZoneMinder::Database::dbh->ping();
 
-    $ZoneMinder::Database::dbh->begin_work();
-    #$event->lock_and_load();
+    my $in_transaction = $ZoneMinder::Database::dbh->{AutoCommit} ? 0 : 1;
 
-    ZoneMinder::Database::zmDbDo('DELETE FROM Frames WHERE EventId=?', $$event{Id});
-    if ( $ZoneMinder::Database::dbh->errstr() ) {
-      $ZoneMinder::Database::dbh->commit();
-      return;
-    }
+    $ZoneMinder::Database::dbh->begin_work() if ! $in_transaction;
+
+    # Going to delete in order of least value to greatest value. Stats is least and references Frames
     ZoneMinder::Database::zmDbDo('DELETE FROM Stats WHERE EventId=?', $$event{Id});
     if ( $ZoneMinder::Database::dbh->errstr() ) {
-      $ZoneMinder::Database::dbh->commit();
+      $ZoneMinder::Database::dbh->commit() if ! $in_transaction;
+      return;
+    }
+    ZoneMinder::Database::zmDbDo('DELETE FROM Frames WHERE EventId=?', $$event{Id});
+    if ( $ZoneMinder::Database::dbh->errstr() ) {
+      $ZoneMinder::Database::dbh->commit() if ! $in_transaction;
       return;
     }
 
     # Do it individually to avoid locking up the table for new events
     ZoneMinder::Database::zmDbDo('DELETE FROM Events WHERE Id=?', $$event{Id});
-    $ZoneMinder::Database::dbh->commit();
+    $ZoneMinder::Database::dbh->commit() if ! $in_transaction;
   }
 
   if ( ( $in_zmaudit or (!$Config{ZM_OPT_FAST_DELETE})) and $event->Storage()->DoDelete() ) {
@@ -402,6 +414,11 @@ sub delete {
 sub delete_files {
   my $event = shift;
 
+  if ( !$event->canEdit() ) {
+    Warning('No permission to delete event.');
+    return 'No permission to delete event.';
+  }
+
   foreach my $Storage (
     @_ ? ($_[0]) : (
       new ZoneMinder::Storage($$event{StorageId}),
@@ -570,6 +587,11 @@ sub DiskSpace {
 sub CopyTo {
   my ( $self, $NewStorage ) = @_;
 
+  if ( !$self->canEdit() ) {
+    Warning('No permission to copy event.');
+    return 'No permission to copy event.';
+  }
+
   my $OldStorage = $self->Storage(undef);
   my ( $OldPath ) = ( $self->Path() =~ /^(.*)$/ ); # De-taint
   if ( ! -e $OldPath ) {
@@ -734,8 +756,13 @@ sub CopyTo {
 } # end sub CopyTo
 
 sub MoveTo {
-
   my ( $self, $NewStorage ) = @_;
+
+  if ( !$self->canEdit() ) {
+    Warning('No permission to move event.');
+    return 'No permission to move event.';
+  }
+
   my $OldStorage = $self->Storage(undef);
 
   my $error = $self->CopyTo($NewStorage);
@@ -794,15 +821,15 @@ sub recover_timestamps {
 
     my $duration = $last_timestamp - $first_timestamp;
     $Event->Length($duration);
-    $Event->StartTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp) );
+    $Event->StartDateTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp) );
     if ( $Event->Scheme() eq 'Deep' and $Event->RelativePath(undef) and ($path ne $Event->Path(undef)) ) {
       my ( $year, $month, $day, $hour, $minute, $second ) =
       ($path =~ /(\d{2})\/(\d{2})\/(\d{2})\/(\d{2})\/(\d{2})\/(\d{2})$/);
       Error("Updating starttime to $path $year/$month/$day $hour:$minute:$second");
-      $Event->StartTime(sprintf('%.4d-%.2d-%.2d %.2d:%.2d:%.2d', 2000+$year, $month, $day, $hour, $minute, $second));
+      $Event->StartDateTime(sprintf('%.4d-%.2d-%.2d %.2d:%.2d:%.2d', 2000+$year, $month, $day, $hour, $minute, $second));
     }
-    $Event->EndTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $last_timestamp) );
-    Debug("From capture Jpegs have duration $duration = $last_timestamp - $first_timestamp : $$Event{StartTime} to $$Event{EndTime}");
+    $Event->EndDateTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $last_timestamp) );
+    Debug("From capture Jpegs have duration $duration = $last_timestamp - $first_timestamp : $$Event{StartDateTime} to $$Event{EndDateTime}");
     $ZoneMinder::Database::dbh->begin_work();
     foreach my $jpg ( @capture_jpgs ) {
       my ( $id ) = $jpg =~ /^(\d+)\-capture\.jpg$/;
@@ -838,8 +865,8 @@ sub recover_timestamps {
       }
     my $seconds = ($h*60*60)+($m*60)+$s;
     $Event->Length($seconds.'.'.$u);
-    $Event->StartTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp) );
-    $Event->EndTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp+$seconds) );
+    $Event->StartDateTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp) );
+    $Event->EndDateTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp+$seconds) );
   }
   if ( @mp4_files ) {
     $Event->DefaultVideo($mp4_files[0]);
@@ -863,16 +890,41 @@ sub files {
 
 sub has_capture_jpegs {
 	@{$_[0]{capture_jpegs}} = grep(/^\d+\-capture\.jpg$/, $_[0]->files());
-	Debug("have " . @{$_[0]{capture_jpegs}} . " capture jpegs");
+	Debug('have ' . @{$_[0]{capture_jpegs}} . ' capture jpegs');
 	return @{$_[0]{capture_jpegs}} ? 1 : 0;
 }
 
 sub has_analyse_jpegs {
 	@{$_[0]{analyse_jpegs}} = grep(/^\d+\-analyse\.jpg$/, $_[0]->files());
-	Debug("have " . @{$_[0]{analyse_jpegs}} . " analyse jpegs");
+	Debug('have ' . @{$_[0]{analyse_jpegs}} . ' analyse jpegs');
 	return @{$_[0]{analyse_jpegs}} ? 1 : 0;
 }
 
+sub canEdit {
+  my $self = shift;
+  if ( !$ZoneMinder::user ) {
+    # No user loaded... assume running as system
+    return 1;
+  }
+  if ( !$$ZoneMinder::user{MonitorIds} ) {
+    # User has no monitor limitations
+    return 1;
+  }
+  if ( $$ZoneMinder::user{Events} eq 'Edit' ) {
+    return 1; 
+  }
+  return 0;
+} # end sub canEdit
+
+sub Monitor {
+  my $self = shift;
+  $$self{Monitor} = shift if @_;
+  if ( !$$self{Monitor} ) {
+    $$self{Monitor} = new ZoneMinder::Monitor($$self{MonitorId});
+  }
+  return $$self{Monitor};
+}
+
 1;
 __END__
 
