@@ -1,6 +1,6 @@
 # ==========================================================================
 #
-# ZoneMinder Filter Module, $Date$, $Revision$
+# ZoneMinder Filter Module
 # Copyright (C) 2001-2008  Philip Coombes
 #
 # This program is free software; you can redistribute it and/or
@@ -49,54 +49,48 @@ use ZoneMinder::Database qw(:all);
 require ZoneMinder::Storage;
 require ZoneMinder::Server;
 
-sub Name {
-  if ( @_ > 1 ) {
-    $_[0]{Name} = $_[1];
-  }
-  return $_[0]{Name};
-} # end sub Path
+use vars qw/ $table $primary_key %fields /;
+$table = 'Users';
+$primary_key = 'Id';
 
-sub find {
-  shift if $_[0] eq 'ZoneMinder::Filter';
-  my %sql_filters = @_;
-
-  my $sql = 'SELECT * FROM Filters';
-  my @sql_filters;
-  my @sql_values;
-
-  if ( exists $sql_filters{Name} ) {
-    push @sql_filters , ' Name = ? ';
-    push @sql_values, $sql_filters{Name};
-  }
-
-  $sql .= ' WHERE ' . join(' AND ', @sql_filters ) if @sql_filters;
-  $sql .= ' LIMIT ' . $sql_filters{limit} if $sql_filters{limit};
-
-  my $sth = $ZoneMinder::Database::dbh->prepare_cached($sql)
-    or Fatal("Can't prepare '$sql': ".$ZoneMinder::Database::dbh->errstr());
-  my $res = $sth->execute(@sql_values)
-    or Fatal("Can't execute '$sql': ".$sth->errstr());
-
-  my @results;
-
-  while( my $db_filter = $sth->fetchrow_hashref() ) {
-    my $filter = new ZoneMinder::Filter($$db_filter{Id}, $db_filter);
-    push @results, $filter;
-  } # end while
-  $sth->finish();
-
-  return @results;
-}
-
-sub find_one {
-  my @results = find(@_);
-  return $results[0] if @results;
-}
+%fields = map { $_ => $_ } qw(
+Id
+Name
+Query_json
+AutoArchive
+AutoUnarchive
+AutoVideo
+AutoUpload
+AutoEmail
+EmailTo
+EmailSubject
+EmailBody
+AutoMessage
+AutoExecute
+AutoExecuteCmd
+AutoDelete
+AutoMove
+AutoMoveTo
+AutoCopy
+AutoCopyTo
+UpdateDiskSpace
+UserId
+Background
+Concurrent
+LockRows
+);
 
 sub Execute {
   my $self = $_[0];
   my $sql = $self->Sql(undef);
 
+  if ( $$self{PreSQLConditions} and @{$$self{PreSQLConditions}} ) {
+    foreach my $term ( @{$$self{PreSQLConditions}} ) {
+      if ( $$term{attr} eq 'DiskPercent' ) {
+      }
+    }
+  }
+
   if ( $self->{HasDiskPercent} ) {
     $$self{Storage} = ZoneMinder::Storage->find_one() if ! $$self{Storage};
 		my $disk_percent = getDiskPercent($$self{Storage} ? $$self{Storage}->Path() : $Config{ZM_DIR_EVENTS});
@@ -112,6 +106,8 @@ sub Execute {
     $sql =~ s/zmSystemLoad/$load/g;
   }
 
+  $sql .= ' FOR UPDATE' if $$self{LockRows};
+
   Debug("Filter::Execute SQL ($sql)");
   my $sth = $ZoneMinder::Database::dbh->prepare_cached($sql)
     or Fatal("Can't prepare '$sql': ".$ZoneMinder::Database::dbh->errstr());
@@ -121,14 +117,29 @@ sub Execute {
     return;
   }
   my @results;
-
   while ( my $event = $sth->fetchrow_hashref() ) {
     push @results, $event;
   }
   $sth->finish();
-  Debug('Loaded ' . @results . " events for filter $_[0]{Name} using query ($sql)");
+  Debug('Loaded ' . @results . ' events for filter '.$$self{Name}.' using query ('.$sql.')"');
+  if ( $self->{PostSQLConditions} ) {
+    my @filtered_events;
+    foreach my $term ( @{$$self{PostSQLConditions}} ) {
+      if ( $$term{attr} eq 'ExistsInFileSystem' ) {
+        foreach my $row ( @results ) {
+          my $event = new ZoneMinder::Event($$row{Id}, $row);
+          if ( -e $event->Path() ) {
+            push @filtered_events, $row if $$term{val} eq 'true';
+          } else {
+            push @filtered_events, $row if $$term{val} eq 'false';
+          }
+        }
+      }
+    } # end foreach term
+    @results = @filtered_events;
+  } # end if has PostSQLConditions
   return @results;
-}
+} # end sub Execute
 
 sub Sql {
   my $self = shift;
@@ -141,77 +152,89 @@ sub Sql {
     }
 
     my $filter_expr = ZoneMinder::General::jsonDecode($self->{Query_json});
-    my $sql = 'SELECT E.*,
-       unix_timestamp(E.StartTime) as Time,
-       M.Name as MonitorName,
-       M.DefaultRate,
-       M.DefaultScale
-         FROM Events as E
-         INNER JOIN Monitors as M on M.Id = E.MonitorId
-         LEFT JOIN Storage as S on S.Id = E.StorageId
-         ';
+    my $sql = 'SELECT E.*, unix_timestamp(E.StartDateTime) as Time
+         FROM Events as E';
 
     if ( $filter_expr->{terms} ) {
       foreach my $term ( @{$filter_expr->{terms}} ) {
 
         # See getFilterQueryConjunctionTypes()
         if ( exists($term->{cnj}) and $term->{cnj} =~ /^(and|or)$/ ) {
-          $self->{Sql} .= ' '.$term->{cnj}.' ';
+          $self->{Sql} .= ' '.$term->{cnj};
         }
+        $self->{Sql} .= ' ';
         if ( exists($term->{obr}) ) {
-          $self->{Sql} .= ' '.str_repeat('(', $term->{obr}).' ';
+          $self->{Sql} .= str_repeat('(', $term->{obr}).' ';
         }
+        if (!$term->{attr}) {
+          Error("Invalid term in filter $$self{Id}. Empty attr");
+          next;
+        }
+
         my $value = $term->{val};
         my @value_list;
-        if ( $term->{attr} ) {
-          if ( $term->{attr} =~ /^Monitor/ ) {
-            my ( $temp_attr_name ) = $term->{attr} =~ /^Monitor(.+)$/;
-            $self->{Sql} .= 'M.'.$temp_attr_name;
-          } elsif ( $term->{attr} eq 'ServerId' or $term->{attr} eq 'MonitorServerId' ) {
-            $self->{Sql} .= 'M.ServerId';
-          } elsif ( $term->{attr} eq 'StorageServerId' ) {
-            $self->{Sql} .= 'S.ServerId';
-          } elsif ( $term->{attr} eq 'FilterServerId' ) {
-            $self->{Sql} .= $Config{ZM_SERVER_ID};
-# StartTime options
-          } elsif ( $term->{attr} eq 'DateTime' ) {
-            $self->{Sql} .= 'E.StartTime';
-          } elsif ( $term->{attr} eq 'StartDateTime' ) {
-            $self->{Sql} .= 'E.StartTime';
-          } elsif ( $term->{attr} eq 'Date' ) {
-            $self->{Sql} .= 'to_days( E.StartTime )';
-          } elsif ( $term->{attr} eq 'StartDate' ) {
-            $self->{Sql} .= 'to_days( E.StartTime )';
-          } elsif ( $term->{attr} eq 'Time' or $term->{attr} eq 'StartTime' ) {
-            $self->{Sql} .= 'extract( hour_second from E.StartTime )';
-          } elsif ( $term->{attr} eq 'Weekday' or $term->{attr} eq 'StartWeekday' ) {
-            $self->{Sql} .= 'weekday( E.StartTime )';
 
-# EndTIme options
-          } elsif ( $term->{attr} eq 'EndDateTime' ) {
-            $self->{Sql} .= 'E.EndTime';
-          } elsif ( $term->{attr} eq 'EndDate' ) {
-            $self->{Sql} .= 'to_days( E.EndTime )';
-          } elsif ( $term->{attr} eq 'EndTime' ) {
-            $self->{Sql} .= 'extract( hour_second from E.EndTime )';
-          } elsif ( $term->{attr} eq 'EndWeekday' ) {
-            $self->{Sql} .= "weekday( E.EndTime )";
-          } elsif ( $term->{attr} eq 'DiskPercent' ) {
-            $self->{Sql} .= 'zmDiskPercent';
-            $self->{HasDiskPercent} = !undef;
-          } elsif ( $term->{attr} eq 'DiskBlocks' ) {
-            $self->{Sql} .= 'zmDiskBlocks';
-            $self->{HasDiskBlocks} = !undef;
-          } elsif ( $term->{attr} eq 'SystemLoad' ) {
-            $self->{Sql} .= 'zmSystemLoad';
-            $self->{HasSystemLoad} = !undef;
-          } else {
-            $self->{Sql} .= 'E.'.$term->{attr};
-          }
+        if ( $term->{attr} eq 'AlarmedZoneId' ) {
+          $term->{op} = 'EXISTS';
+        } elsif ( $term->{attr} =~ /^Monitor/ ) {
+          $sql = 'SELECT E.*, unix_timestamp(E.StartDateTime) as Time, M.Name as MonitorName
+          FROM Events as E INNER JOIN Monitors as M on M.Id = E.MonitorId';
+          my ( $temp_attr_name ) = $term->{attr} =~ /^Monitor(.+)$/;
+          $self->{Sql} .= 'M.'.$temp_attr_name;
+        } elsif ( $term->{attr} eq 'ServerId' or $term->{attr} eq 'MonitorServerId' ) {
+          $sql = 'SELECT E.*, unix_timestamp(E.StartDateTime) as Time, M.Name as MonitorName
+          FROM Events as E INNER JOIN Monitors as M on M.Id = E.MonitorId';
+          $self->{Sql} .= 'M.ServerId';
+        } elsif ( $term->{attr} eq 'StorageServerId' ) {
+          $self->{Sql} .= '(SELECT Storage.ServerId FROM Storage WHERE Storage.Id=E.StorageId)';
+        } elsif ( $term->{attr} eq 'FilterServerId' ) {
+          $self->{Sql} .= $Config{ZM_SERVER_ID};
+          # StartTime options
+        } elsif ( $term->{attr} eq 'DateTime' ) {
+          $self->{Sql} .= 'E.StartDateTime';
+        } elsif ( $term->{attr} eq 'Date' ) {
+          $self->{Sql} .= 'to_days( E.StartDateTime )';
+        } elsif ( $term->{attr} eq 'StartDate' ) {
+          $self->{Sql} .= 'to_days( E.StartDateTime )';
+        } elsif ( $term->{attr} eq 'Time' or $term->{attr} eq 'StartTime' ) {
+          $self->{Sql} .= 'extract( hour_second from E.StartDateTime )';
+        } elsif ( $term->{attr} eq 'Weekday' or $term->{attr} eq 'StartWeekday' ) {
+          $self->{Sql} .= 'weekday( E.StartDateTime )';
 
+          # EndTIme options
+        } elsif ( $term->{attr} eq 'EndDateTime' ) {
+          $self->{Sql} .= 'E.EndDateTime';
+        } elsif ( $term->{attr} eq 'EndDate' ) {
+          $self->{Sql} .= 'to_days( E.EndDateTime )';
+        } elsif ( $term->{attr} eq 'EndTime' ) {
+          $self->{Sql} .= 'extract( hour_second from E.EndDateTime )';
+        } elsif ( $term->{attr} eq 'EndWeekday' ) {
+          $self->{Sql} .= 'weekday( E.EndDateTime )';
+        } elsif ( $term->{attr} eq 'ExistsInFileSystem' ) {
+          push @{$self->{PostSQLConditions}}, $term;
+          $self->{Sql} .= 'TRUE /* ExistsInFileSystem */';
+        } elsif ( $term->{attr} eq 'DiskPercent' ) {
+          $self->{Sql} .= 'zmDiskPercent';
+          $self->{HasDiskPercent} = !undef;
+        } elsif ( $term->{attr} eq 'DiskBlocks' ) {
+          $self->{Sql} .= 'zmDiskBlocks';
+          $self->{HasDiskBlocks} = !undef;
+        } elsif ( $term->{attr} eq 'SystemLoad' ) {
+          $self->{Sql} .= 'zmSystemLoad';
+          $self->{HasSystemLoad} = !undef;
+        } else {
+          $self->{Sql} .= 'E.'.$term->{attr};
+        }
+
+        if ( $term->{attr} eq 'ExistsInFileSystem' ) {
+          # PostCondition, so no further SQL
+        } else {
           ( my $stripped_value = $value ) =~ s/^["\']+?(.+)["\']+?$/$1/;
           foreach my $temp_value ( split( /["'\s]*?,["'\s]*?/, $stripped_value ) ) {
-            if ( $term->{attr} =~ /^MonitorName/ ) {
+
+            if ( $term->{attr} eq 'AlarmedZoneId' ) {
+              $value = '(SELECT * FROM Stats WHERE EventId=E.Id AND Score > 0 AND ZoneId='.$value.')';
+            } elsif ( $term->{attr} =~ /^MonitorName/ ) {
               $value = "'$temp_value'";
             } elsif ( $term->{attr} =~ /ServerId/) {
               Debug("ServerId, temp_value is ($temp_value) ($ZoneMinder::Config::Config{ZM_SERVER_ID})");
@@ -226,18 +249,18 @@ sub Sql {
                 # This gets used later, I forget for what
                 $$self{Server} = new ZoneMinder::Server($temp_value);
               }
-						} elsif ( $term->{attr} eq 'StorageId' ) {
-							$value = "'$temp_value'";
-							$$self{Storage} = new ZoneMinder::Storage($temp_value);
+            } elsif ( $term->{attr} eq 'StorageId' ) {
+              $value = "'$temp_value'";
+              $$self{Storage} = new ZoneMinder::Storage($temp_value);
             } elsif ( $term->{attr} eq 'Name'
-                || $term->{attr} eq 'Cause'
-                || $term->{attr} eq 'Notes'
-                ) {
-                if ( $term->{op} eq 'LIKE'
+              || $term->{attr} eq 'Cause'
+              || $term->{attr} eq 'Notes'
+            ) {
+              if ( $term->{op} eq 'LIKE'
                 || $term->{op} eq 'NOT LIKE'
-                ) {
+              ) {
                 $temp_value = '%'.$temp_value.'%' if $temp_value !~ /%/;
-                }
+              }
               $value = "'$temp_value'";
             } elsif ( $term->{attr} eq 'DateTime' or $term->{attr} eq 'StartDateTime' or $term->{attr} eq 'EndDateTime' ) {
               if ( $temp_value eq 'NULL' ) {
@@ -253,6 +276,8 @@ sub Sql {
             } elsif ( $term->{attr} eq 'Date' or $term->{attr} eq 'StartDate' or $term->{attr} eq 'EndDate' ) {
               if ( $temp_value eq 'NULL' ) {
                 $value = $temp_value;
+              } elsif ( $temp_value eq 'CURDATE()' or $temp_value eq 'NOW()' ) {
+                $value = 'to_days('.$temp_value.')';
               } else {
                 $value = DateTimeToSQL($temp_value);
                 if ( !$value ) {
@@ -277,55 +302,54 @@ sub Sql {
             }
             push @value_list, $value;
           } # end foreach temp_value
-        } # end if has an attr
 
-        if ( $term->{op} ) {
-          if ( $term->{op} eq '=~' ) {
-            $self->{Sql} .= ' regexp '.$value;
-          } elsif ( $term->{op} eq '!~' ) {
-            $self->{Sql} .= ' not regexp '.$value;
-          } elsif ( $term->{op} eq 'IS' ) {
-            if ( $value eq 'Odd' ) {
-              $self->{Sql} .= ' % 2 = 1';
-            } elsif ( $value eq 'Even' ) {
-              $self->{Sql} .= ' % 2 = 0';
+          if ( $term->{op} ) {
+            if ( $term->{op} eq '=~' ) {
+              $self->{Sql} .= ' REGEXP '.$value;
+            } elsif ( $term->{op} eq '!~' ) {
+              $self->{Sql} .= ' NOT REGEXP '.$value;
+            } elsif ( $term->{op} eq 'IS' ) {
+              if ( $value eq 'Odd' ) {
+                $self->{Sql} .= ' % 2 = 1';
+              } elsif ( $value eq 'Even' ) {
+                $self->{Sql} .= ' % 2 = 0';
+              } else {
+                $self->{Sql} .= ' IS '.$value;
+              }
+            } elsif ( $term->{op} eq 'EXISTS' ) {
+              $self->{Sql} .= ' EXISTS '.$value;
+            } elsif ( $term->{op} eq 'IS NOT' ) {
+              $self->{Sql} .= ' IS NOT '.$value;
+            } elsif ( $term->{op} eq '=[]' or $term->{op} eq 'IN' ) {
+              $self->{Sql} .= ' IN ('.join(',', @value_list).")";
+            } elsif ( $term->{op} eq '![]' ) {
+              $self->{Sql} .= ' NOT IN ('.join(',', @value_list).')';
+            } elsif ( $term->{op} eq 'LIKE' ) {
+              $self->{Sql} .= ' LIKE '.$value;
+            } elsif ( $term->{op} eq 'NOT LIKE' ) {
+              $self->{Sql} .= ' NOT LIKE '.$value;
             } else {
-              $self->{Sql} .= ' IS '.$value;
+              $self->{Sql} .= ' '.$term->{op}.' '.$value;
             }
-          } elsif ( $term->{op} eq 'IS NOT' ) {
-            $self->{Sql} .= ' IS NOT '.$value;
-          } elsif ( $term->{op} eq '=[]' or $term->{op} eq 'IN' ) {
-            $self->{Sql} .= ' IN ('.join(',', @value_list).')';
-          } elsif ( $term->{op} eq '![]' ) {
-            $self->{Sql} .= ' NOT IN ('.join(',', @value_list).')';
-          } elsif ( $term->{op} eq 'LIKE' ) {
-            $self->{Sql} .= ' LIKE '.$value;
-          } elsif ( $term->{op} eq 'NOT LIKE' ) {
-            $self->{Sql} .= ' NOT LIKE '.$value;
-          } else {
-            $self->{Sql} .= ' '.$term->{op}.' '.$value;
-          }
-        } # end if has an operator
-        if ( exists($term->{cbr}) ) {
-          $self->{Sql} .= ' '.str_repeat(')', $term->{cbr}).' ';
-        }
+          } # end if has an operator
+        } # end if Pre/Post or SQL
+        $self->{Sql} .= ' '.str_repeat(')', $term->{cbr}) if exists($term->{cbr});
+        $self->{Sql} .= "\n";
       } # end foreach term
     } # end if terms
 
     if ( $self->{Sql} ) {
-      if ( $self->{AutoMessage} ) {
 # Include all events, including events that are still ongoing
 # and have no EndTime yet
-        $sql .= ' WHERE ( '.$self->{Sql}.' )';
-      } else {
-# Only include closed events (events with valid EndTime)
-        $sql .= ' WHERE (E.EndTime IS NOT NULL) AND ( '.$self->{Sql}.' )';
-      }
+      $sql .= ' WHERE ( '.$self->{Sql}.' )';
     }
     my @auto_terms;
     if ( $self->{AutoArchive} ) {
       push @auto_terms, 'E.Archived = 0';
     }
+    if ( $self->{AutoUnarchive} ) {
+      push @auto_terms, 'E.Archived = 1';
+    }
     # Don't do this, it prevents re-generation and concatenation.
     # If the file already exists, then the video won't be re-recreated
     if ( $self->{AutoVideo} ) {
@@ -347,20 +371,26 @@ sub Sql {
       $sql .= ' AND ( '.join(' or ', @auto_terms).' )';
     }
     if ( !$filter_expr->{sort_field} ) {
-      $filter_expr->{sort_field} = 'StartTime';
+      $filter_expr->{sort_field} = 'StartDateTime';
       $filter_expr->{sort_asc} = 0;
     }
     my $sort_column = '';
     if ( $filter_expr->{sort_field} eq 'Id' ) {
       $sort_column = 'E.Id';
     } elsif ( $filter_expr->{sort_field} eq 'MonitorName' ) {
+            $sql = 'SELECT E.*, unix_timestamp(E.StartDateTime) as Time, M.Name as MonitorName
+         FROM Events as E INNER JOIN Monitors as M on M.Id = E.MonitorId';
       $sort_column = 'M.Name';
     } elsif ( $filter_expr->{sort_field} eq 'Name' ) {
       $sort_column = 'E.Name';
+    } elsif ( $filter_expr->{sort_field} eq 'StartDateTime' ) {
+      $sort_column = 'E.StartDateTime';
     } elsif ( $filter_expr->{sort_field} eq 'StartTime' ) {
-      $sort_column = 'E.StartTime';
+      $sort_column = 'E.StartDateTime';
     } elsif ( $filter_expr->{sort_field} eq 'EndTime' ) {
-      $sort_column = 'E.EndTime';
+      $sort_column = 'E.EndDateTime';
+    } elsif ( $filter_expr->{sort_field} eq 'EndDateTime' ) {
+      $sort_column = 'E.EndDateTime';
     } elsif ( $filter_expr->{sort_field} eq 'Secs' ) {
       $sort_column = 'E.Length';
     } elsif ( $filter_expr->{sort_field} eq 'Frames' ) {
@@ -376,7 +406,7 @@ sub Sql {
     } elsif ( $filter_expr->{sort_field} eq 'DiskSpace' ) {
       $sort_column = 'E.DiskSpace';
     } else {
-      $sort_column = 'E.StartTime';
+      $sort_column = 'E.StartDateTime';
     }
     my $sort_order = $filter_expr->{sort_asc} ? 'ASC' : 'DESC';
     $sql .= ' ORDER BY '.$sort_column.' '.$sort_order;
@@ -447,6 +477,15 @@ sub DateTimeToSQL {
   return POSIX::strftime('%Y-%m-%d %H:%M:%S', localtime($dt_val));
 }
 
+sub User {
+  my $self = shift;
+  $$self{User} = shift if @_;
+  if ( ! $$self{User} and $$self{UserId} ) {
+    $$self{User} = ZoneMinder::User->find_one(Id=>$$self{UserId});
+  }
+  return $$self{User};
+}
+
 1;
 __END__
 # Below is stub documentation for your module. You'd better edit it!
