@@ -58,6 +58,14 @@ case $i in
     PACKAGE_VERSION="${i#*=}"
     shift
     ;;
+    -x=*|--debbuild-extra=*)
+    DEBBUILD_EXTRA="${i#*=}"
+    shift
+    ;;
+    --dput=*)
+    DPUT="${i#*=}"
+    shift
+    ;;
     --default)
     DEFAULT=YES
     shift # past argument with no value
@@ -80,7 +88,7 @@ fi;
 
 if [ "$DISTROS" == "" ]; then
   if [ "$RELEASE" != "" ]; then
-    DISTROS="xenial,bionic,focal,trusty"
+    DISTROS="xenial,bionic,focal,hirsute,impish"
   else
     DISTROS=`lsb_release -a 2>/dev/null | grep Codename | awk '{print $2}'`;
   fi;
@@ -110,7 +118,17 @@ else
   fi;
   if [ "$SNAPSHOT" == "stable" ]; then
     if [ "$BRANCH" == "" ]; then
-      BRANCH=$(git describe --tags $(git rev-list --tags --max-count=1));
+      #REV=$(git rev-list --tags --max-count=1)
+      BRANCH=`git describe --tags $(git rev-list --tags --max-count=1)`;
+      if [ -z "$BRANCH" ]; then
+        # This should only happen in CI environments where tag info isn't available
+        BRANCH=`cat version`
+        echo "Building branch $BRANCH"
+      fi
+      if [ "$BRANCH" == "" ]; then
+        echo "Unable to determine latest stable branch!"
+        exit 0;
+      fi
       echo "Latest stable branch is $BRANCH";
     fi;
   else
@@ -151,10 +169,14 @@ if [ ! -d "${GITHUB_FORK}_zoneminder_release" ]; then
   if [ -d "${GITHUB_FORK}_ZoneMinder.git" ]; then
     echo "Using local clone ${GITHUB_FORK}_ZoneMinder.git to pull from."
     cd "${GITHUB_FORK}_ZoneMinder.git"
-    echo "git pull..."
-    git pull
+    echo "git fetch..."
+    git fetch
     echo "git checkout $BRANCH"
     git checkout $BRANCH
+    if [ $? -ne 0 ]; then
+      echo "Failed to switch to branch."
+      exit 1;
+    fi;
     echo "git pull..."
     git pull
     cd ../
@@ -207,6 +229,7 @@ rm -rf .git
 rm .gitignore
 cd ../
 
+
 if [ ! -e "$DIRECTORY.orig.tar.gz" ]; then
   tar zcf $DIRECTORY.orig.tar.gz $DIRECTORY.orig
 fi;
@@ -220,14 +243,8 @@ IFS=',' ;for DISTRO in `echo "$DISTROS"`; do
   fi;
 
   # Generate Changlog
-  if [ "$DISTRO" == "trusty" ] || [ "$DISTRO" == "precise" ]
-  then
-    cp -Rpd distros/ubuntu1204 debian
-
-  elif [ "$DISTRO" == "wheezy" ]
-  then
-    cp -Rpd distros/debian debian
-
+  if [ "$DISTRO" == "focal" ] || [ "$DISTRO" == "buster" ] || [ "$DISTRO" == "hirsute" ] || [ "$DISTRO" == "impish" ]; then 
+    cp -Rpd distros/ubuntu2004 debian
   elif [ "$DISTRO" == "beowulf" ]
   then
     cp -Rpd distros/beowulf debian
@@ -282,30 +299,37 @@ zoneminder ($VERSION-$DISTRO${PACKAGE_VERSION}) $DISTRO; urgency=$URGENCY
 EOF
   fi;
 
+  # Leave the .orig so that we don't pollute it when building deps
+  cd ..
   if [ $TYPE == "binary" ]; then
-    # Auto-install all ZoneMinder's depedencies using the Debian control file
-    sudo apt-get install devscripts equivs
-    sudo mk-build-deps -ir ./debian/control
-    echo "Status: $?"
-    DEBUILD=debuild
+	  # Auto-install all ZoneMinder's depedencies using the Debian control file
+	  sudo apt-get install devscripts equivs
+	  sudo mk-build-deps -ir $DIRECTORY.orig/debian/control
+	  echo "Status: $?"
+	  DEBUILD=debuild
   else
-    if [ $TYPE == "local" ]; then
-      # Auto-install all ZoneMinder's depedencies using the Debian control file
-      sudo apt-get install devscripts equivs
-      sudo mk-build-deps -ir ./debian/control
-      echo "Status: $?"
-      DEBUILD="debuild -i -us -uc -b"
-    else 
-      # Source build, don't need build depends.
-      DEBUILD="debuild -S -sa"
-    fi;
+	  if [ $TYPE == "local" ]; then
+		  # Auto-install all ZoneMinder's depedencies using the Debian control file
+		  sudo apt-get install devscripts equivs
+		  sudo mk-build-deps -ir $DIRECTORY.orig/debian/control
+		  echo "Status: $?"
+		  DEBUILD="debuild -i -us -uc -b"
+	  else 
+		  # Source build, don't need build depends.
+		  DEBUILD="debuild -S -sa"
+	  fi;
   fi;
+
+  cd $DIRECTORY.orig
+
   if [ "$DEBSIGN_KEYID" != "" ]; then
     DEBUILD="$DEBUILD -k$DEBSIGN_KEYID"
   fi
+  # Add any extra options specified on the CLI
+  DEBUILD="$DEBUILD $DEBBUILD_EXTRA"
   eval $DEBUILD
   if [ $? -ne 0 ]; then
-  echo "Error status code is: $?"
+    echo "Error status code is: $?"
     echo "Build failed.";
     exit $?;
   fi;
@@ -341,8 +365,10 @@ EOF
         dput $PPA $SC
       fi;
     else
-      echo "dputting to $PPA";
-      dput $PPA $SC
+      if [ "$DPUT" != "no" ]; then
+        echo "dputting to $PPA";
+        dput $PPA $SC
+      fi;
     fi;
   fi;
 done; # foreach distro
