@@ -14,8 +14,8 @@ class Event extends ZM_Object {
     'StorageId' => null,
     'SecondaryStorageId' => null,
     'Cause' => '',
-    'StartTime' => null,
-    'EndTime' => null,
+    'StartDateTime' => null,
+    'EndDateTime' => null,
     'Width' => null,
     'Height' => null,
     'Length' => null,
@@ -50,6 +50,9 @@ class Event extends ZM_Object {
   public static function clear_cache() {
     return ZM_Object::_clear_cache(get_class());
   }
+  public function remove_from_cache() {
+    return ZM_Object::_remove_from_cache(get_class(), $this);
+  }
 
   public function Storage( $new = null ) {
     if ( $new ) {
@@ -58,8 +61,10 @@ class Event extends ZM_Object {
     if ( ! ( property_exists($this, 'Storage') and $this->{'Storage'} ) ) {
       if ( isset($this->{'StorageId'}) and $this->{'StorageId'} )
         $this->{'Storage'} = Storage::find_one(array('Id'=>$this->{'StorageId'}));
-      if ( ! ( property_exists($this, 'Storage') and $this->{'Storage'} ) )
+      if ( ! ( property_exists($this, 'Storage') and $this->{'Storage'} ) ) {
         $this->{'Storage'} = new Storage(NULL);
+        $this->{'Storage'}->Scheme($this->Scheme());
+      }
     }
     return $this->{'Storage'};
   }
@@ -88,7 +93,7 @@ class Event extends ZM_Object {
 
   public function Time() {
     if ( ! isset($this->{'Time'}) ) {
-      $this->{'Time'} = strtotime($this->{'StartTime'});
+      $this->{'Time'} = strtotime($this->{'StartDateTime'});
     }
     return $this->{'Time'};
   }
@@ -130,6 +135,10 @@ class Event extends ZM_Object {
       Error('Event delete on event with empty Id');
       return;
     }
+    if ( $this->{'Archived'} ) {
+      Error('Cannot delete an Archived event.');
+      return;
+    }
     if ( ZM_OPT_FAST_DELETE ) {
       dbQuery('DELETE FROM Events WHERE Id = ?', array($this->{'Id'}));
       return;
@@ -144,9 +153,9 @@ class Event extends ZM_Object {
       if ( $this->{'Scheme'} == 'Deep' ) {
 
         # Assumption: All events have a start time
-        $start_date = date_parse($this->{'StartTime'});
+        $start_date = date_parse($this->{'StartDateTime'});
         if ( ! $start_date ) {
-          throw new Exception('Unable to parse start time for event ' . $this->{'Id'} . ' not deleting files.');
+          throw new Exception('Unable to parse start date time for event ' . $this->{'Id'} . ' not deleting files.');
         }
         $start_date['year'] = $start_date['year'] % 100;
 
@@ -270,7 +279,7 @@ class Event extends ZM_Object {
     }
     if ( (!property_exists($this, 'DiskSpace')) or (null === $this->{'DiskSpace'}) ) {
       $this->{'DiskSpace'} = folder_size($this->Path());
-      if ( $this->{'EndTime'} ) {
+      if ( $this->{'EndDateTime'} ) {
         # Finished events shouldn't grow in size much so we can commit it to the db.
         dbQuery('UPDATE Events SET DiskSpace=? WHERE Id=?', array($this->{'DiskSpace'}, $this->{'Id'}));
       }
@@ -282,7 +291,7 @@ class Event extends ZM_Object {
 	# The idea here is that we don't really want to use the analysis jpeg as the thumbnail.  
 	# The snapshot image will be generated during capturing
     if ( file_exists($this->Path().'/snapshot.jpg') ) {
-      Logger::Debug("snapshot exists");
+      Debug("snapshot exists");
       $frame = null;
     } else {
       # Load the frame with the highest score to use as a thumbnail
@@ -387,14 +396,14 @@ class Event extends ZM_Object {
 
     if ( $frame and !is_array($frame) ) {
       # Must be an Id
-      Logger::Debug("Assuming that $frame is an Id");
+      Debug("Assuming that $frame is an Id");
       $frame = array('FrameId'=>$frame, 'Type'=>'', 'Delta'=>0);
     }
 
     if ( ( !$frame ) and file_exists($eventPath.'/snapshot.jpg') ) {
       # No frame specified, so look for a snapshot to use
       $captImage = 'snapshot.jpg';
-      Logger::Debug('Frame not specified, using snapshot');
+      Debug('Frame not specified, using snapshot');
       $frame = array('FrameId'=>'snapshot', 'Type'=>'', 'Delta'=>0);
     } else {
       $captImage = sprintf('%0'.ZM_EVENT_IMAGE_DIGITS.'d-analyze.jpg', $frame['FrameId']);
@@ -412,11 +421,11 @@ class Event extends ZM_Object {
               
             #$command ='ffmpeg -v 0 -i '.$videoPath.' -vf "select=gte(n\\,'.$frame['FrameId'].'),setpts=PTS-STARTPTS" '.$eventPath.'/'.$captImage;
             $command ='ffmpeg -ss '. $frame['Delta'] .' -i '.$videoPath.' -frames:v 1 '.$eventPath.'/'.$captImage;
-            Logger::Debug('Running '.$command);
+            Debug('Running '.$command);
             $output = array();
             $retval = 0;
             exec($command, $output, $retval);
-            Logger::Debug("Retval: $retval, output: " . implode("\n", $output));
+            Debug("Retval: $retval, output: " . implode("\n", $output));
           } else {
             Error('Can\'t create frame images from video because there is no video file for event '.$Event->Id().' at ' .$Event->Path());
           }
@@ -520,7 +529,7 @@ class Event extends ZM_Object {
           return;
         }
       }
-      Logger::Debug("sending command to $url");
+      Debug("sending command to $url");
       // use key 'http' even if you send the request to https://...
       $options = array(
           'http' => array(
@@ -536,7 +545,7 @@ class Event extends ZM_Object {
           Error("Error restarting zmc using $url");
         }
         $event_data = json_decode($result,true);
-        Logger::Debug(print_r($event_data['event']['Event'],1));
+        Debug(print_r($event_data['event']['Event'],1));
         return $event_data['event']['Event']['fileExists'];
       } catch ( Exception $e ) {
         Error("Except $e thrown trying to get event data");
@@ -568,7 +577,7 @@ class Event extends ZM_Object {
           return;
         }
       }
-      Logger::Debug("sending command to $url");
+      Debug("sending command to $url");
       // use key 'http' even if you send the request to https://...
       $options = array(
           'http' => array(
@@ -584,7 +593,7 @@ class Event extends ZM_Object {
           Error("Error restarting zmc using $url");
         }
         $event_data = json_decode($result,true);
-        Logger::Debug(print_r($event_data['event']['Event'], 1));
+        Debug(print_r($event_data['event']['Event'], 1));
         return $event_data['event']['Event']['fileSize'];
       } catch ( Exception $e ) {
         Error("Except $e thrown trying to get event data");
@@ -597,7 +606,7 @@ class Event extends ZM_Object {
     if ( $this->Archived() ) {
       return false;
     }
-    if ( !$this->EndTime() ) {
+    if ( !$this->EndDateTime() ) {
       return false;
     }
     if ( !canEdit('Events') ) {
@@ -610,7 +619,7 @@ class Event extends ZM_Object {
   public function cant_delete_reason() {
     if ( $this->Archived() ) {
       return 'You cannot delete an archived event. Unarchive it first.';
-    } else if ( ! $this->EndTime() ) {
+    } else if ( ! $this->EndDateTime() ) {
       return 'You cannot delete an event while it is being recorded. Wait for it to finish.';
     } else if ( ! canEdit('Events') ) {
       return 'You do not have rights to edit Events.';
@@ -618,6 +627,29 @@ class Event extends ZM_Object {
     return 'Unknown reason';
   }
 
+  function canView($u=null) {
+    global $user;
+    if (!$u) $u=$user;
+    if (!$u) {
+      # auth turned on and not logged in
+      return false;
+    }
+    if (!empty($u['MonitorIds']) ) {
+      if (in_array($this->{'MonitorId'}, explode(',', $u['MonitorIds']))) {
+        return true;
+      }
+      return false;
+    }
+    if ($u['Events'] != 'None') {
+      return true;
+    }
+    if ($u['Snapshots'] != 'None') {
+      # If the event is contained in a snapshot, then we can still view it.
+      if (dbFetchOne('SELECT * FROM Snapshot_Events WHERE EventId=?', $this->Id()))
+        return true;
+    }
+    return false;
+  }
 } # end class
 
 ?>
